use std::path::PathBuf;

use clap::Parser;

use super::{AmbientDriverError, Config, Leaf};
use ambient_driver::project::{ProjectError, Projects};

#[derive(Debug, Parser)]
pub struct ProjectsCmd {
    /// Name of YAML file with projects.
    projects: PathBuf,
}

impl Leaf for ProjectsCmd {
    fn run(&self, _config: &Config) -> Result<(), AmbientDriverError> {
        let projects = Projects::from_file(&self.projects)
            .map_err(|err| ProjectsCmdError::FromFile(self.projects.clone(), err))?;
        let mut names: Vec<&str> = projects.iter().map(|(name, _)| name).collect();
        names.sort();
        for name in names.iter() {
            println!("{}", name);
        }

        Ok(())
    }
}

#[derive(Debug, thiserror::Error)]
pub enum ProjectsCmdError {
    #[error("failed to load project list from file {0}")]
    FromFile(PathBuf, #[source] ProjectError),
}
